![pipeline](https://git.unistra.fr/vue-unistra/cas-authentication/badges/develop/pipeline.svg)
![coverage](https://git.unistra.fr/vue-unistra/cas-authentication/badges/develop/coverage.svg)


# Installation 

```bash
npm i -S @vue-unistra/cas-authentication@^2.1.0
```

Modifier la version après le `@` en fonction de la version souhaitée.

La version `1.x.x` est compatible avec Vue 2.  
La version `2.x.x` est compatible avec Vue 3.

# Configuration 

## Router

Création d'une instance de Vue-Router :

### Vue 2

```javascript
import Vue from 'vue';
import VueRouter from 'vue-router';
import HelloWorld from '@/components/HelloWorld';
import ErrorPage from '@/views/ErroPage'

Vue.use(VueRouter);

let router = new VueRouter({
  mode: 'history',
  routes: [
      {
        path: '/',
        name: 'HelloWorld',
        component: HelloWorld,
        meta: {
          unistraCasAuthentication: true
        }
      },
      {
        path: '/auth/cas/logout',
        name: 'cas_authentication_logout',
      },
      {
        path: '/error/token',
        name: 'vue_cas_token_error',
        component: ErrorPage,
      },
  ],
});
```

### Vue 3 + TypeScript + Vite

```typescript
import { createRouter, createWebHistory, RouteRecordRaw } from 'vue-router';
import HelloWorld from '@/components/HelloWorld.vue';

const routes: Array<RouteRecordRaw> = [
  {
    path: '/',
    name: 'HelloWorld',
    component: HelloWorld,
    meta: {
      unistraCasAuthentication: true
    }
  },
  {
    path: '/auth/cas/logout',
    name: 'cas_authentication_logout',
  },
  {
    path: '/error/token',
    name: 'vue_cas_token_error',
    component: () => import('@/views/ErroPage.vue'),
  },
];

export const router = createRouter({
  history: createWebHistory(import.meta.env.BASE_URL),
  routes,
});
```

### Informations

Le Router doit être obligatoirement être initialisé à `mode: 'history'`.  
Si vous souhaitez utiliser le module CAS de déconnexion, créez au moins une route avec un `path` et un `name`, le name sera à passer dans le `main.[js|ts]`.  
En cas d'erreur lors de la demande de token, le plugin redirige vers la route nommée `vue_cas_token_error`. Si vous n'avez pas créé cette route, le plugin la créera pour vous.  
Les routes qui seront protégées par CAS, doivent avoir la `meta: {unistraCasAuthentication: true}`. 
Cette solution est à privilégier lorsque vous souhaitez avoir des pages qui ne nécessitent pas d'authentification. Si vous souhaitez protéger globalement votre application, préférez l'option `appIsAllAuth` dans le `main.[js|ts]` (cf. le point `Ajout du plugin à Vue` ci-dessous).

## Axios

Création d'une instance d'Axios : 

### Vue 2

```javascript
import Vue from 'vue';
import Axios from 'axios';
import VueAxios from 'vue-axios';

const config = {
    baseURL: process.env.VUE_APP_API_URL,
}

let axios = Axios.create(config);

Vue.use(VueAxios, axios);
```

L'utilisation de Vue-Axios permet d'intégrer plus facilement Axios dans les variables de notre instance Vue.

### Vue 3 + TypeScript + Vite

```typescript
import axios from 'axios';
import { AxiosInstance } from 'axios';

const config = {
    baseURL: import.meta.env.VITE_APP_API_URL,
}

export const axiosInstance: AxiosInstance = axios.create(config);
```

Dans Vue 3 on n'intègre plus axios dans l'instance de Vue.

## Ajout du plugin à Vue

Dans le `main.[js|ts]` ou le `index.[js|ts]` :

### Vue 2

```javascript
import Vue from 'vue';
import App from '@/App.vue';

import { CasAuthentication } from 'vue-cas-authentication';

const CasOptions = {
  router: router,
  axios: axios,
  options: {
    appIsAllAuth: true,
    authCasLogoutUrl: 'cas_authentication_logout',
    jwtServerUrl: 'https://jwtserver-pprd.app.unistra.fr/api',
    serverCAS: 'https://cas.unistra.fr',
    tokenErrorRoutes: {
      /**@type number*/errorCode: /**@type string*/routeName,
    }
  },
};

Vue.use(CasAuthentication, CasOptions);

Vue.config.productionTip = false;

new Vue({
  axios: axios,
  router: router,
  store: store,
  render: h => h(App),
}).$mount('#app');
```

### Vue 3 + TypeScript + Vite

```typescript
import { createApp } from 'vue';
import App from '@/App.vue';

import { CasAuthentication } from 'vue-cas-authentication';
import { router } from '@/router';

import { axiosInstance } from '@/axios';

const app = createApp(App);

const CasOptions = {
  axios: axiosInstance,
  options: {
    appIsAllAuth: true,
    authCasLogoutUrl: 'cas_authentication_logout',
    hasFreeAPI: true,
    jwtServerUrl: 'https://jwtserver-pprd.app.unistra.fr/api',
    loginRoute: {name: 'login'},
    loginRouteIsInternal: true,
    serverCAS: 'https://cas.unistra.fr',
    tokenErrorRoutes: {
      /**@type number*/errorCode: /**@type string*/routeName,
    }
  },
  router: router,
};

app.use(router);
app.use(CasAuthentication, CasOptions);
app.mount('#app');
```

Le plugin va surcharger les instances du Router et d'Axios pour permettre de rajouter des options et des variables spécifiques au projet en cours.  

# Options

Toutes les options sont optionnelles.

## appIsAllAuth

L'option `appIsAllAuth` permet d'indiquer que l'application nécessitera globalement d'une authentification.
Si vous faites le choix de cette solution, la `meta` n'est pas nécessaire au niveau du routeur.

## authCasLogoutUrl

Pour l'attribut `authCasLogoutUrl` il est nécessaire de mettre la même valeur que l'attribut `name` de votre route de déconnexion dans le routeur.

## hasFreeAPI

L'option `hasFreeAPI` permet de définir si votre application comporte des APIs publiques ou non.  
Si vous mettez cette option à `false` ou si vous l'omettez, le plugin ajoutera automatiquement un `interceptor` sur Axios pour ajouter le token JWT dans toutes les requêtes. Cet intercepteur vérifiera également si le token est valide/expiré et le renouvellera si besoin.  

L'intercepteur est disponible avec le composable (cf. le point `Composable` ci-dessous).

## jwtServerUrl

URL du serveur qui gère le token. Cette URL est utilisée pour demander un token lors de la méthode `refresh` ou pour servir de `service` pour le CAS dans la méthode `login` si vous n'avez pas indiqué d'URL dans `loginRoute` (cf. le point `loginRoute` ci-dessous).

## loginRoute

L'attribut `loginRoute` permet de définir une route spécifique pour la connexion. Exemple, si votre système de connexion est géré par une page de votre application (à associer alors avec `loginRouteIsInternal`) ou si elle est gérée par un autre serveur.

Si c'est une route interne à votre application, vous pouvez passer un objet `RouteLocationRaw` (cf. [la doc de Vue Router](https://next.router.vuejs.org/api/#routelocationraw)).  
Si c'est une route externe, vous pouvez passer une URL (string).

## loginRouteIsInternal

L'attribut `loginRouteIsInternal` permet de définir si la route de connexion est interne (URL de l'application front) ou externe (URL vers une autre app ou vers le serveur).

## serverCAS

L'attribut `serverCAS` permet de définir l'URL du serveur CAS.

## tokenErrorRoutes

L'attribut `tokenErrorRoutes` permet de définir des routes spécifiques à appeler selon le code d'erreur donné par le server jwt.  
Par exemple s'il vaut `{403: 'ForbiddenPage'}` le plugin routera vers `ForbiddenPage` si le server jwt renvoie une erreur 403, et vers la route nommée `vue_cas_token_error` pour tout autre code d'erreur.

# Composable

## useCasAuthentication

Le composable `useCasAuthentication` permet d'accéder aux méthodes du plugin. 

```typescript
import { useCasAuthentication } from '@vue-unistra/cas-authentication';

const { login, axiosRequestInterceptor } = useCasAuthentication();
```

### axiosRequestInterceptor

Permet de récupérer directement l'intercepteur Axios que vous pourrez ajouter à votre instance Axios. Cela permet d'avoir une instance d'Axios avec une gestion des JWT et une sans.

La fonction prend comme paramètre un objet avec les attributs suivants :

- `axios` : instance Axios qui permet de faire le `refresh` du token.
- `jwtServerUrl` : URL du serveur qui gère le token (cf. le point `jwtServerUrl` ci-dessus).
- `options`: options du plugin (cf. le point `Options` ci-dessus).
- `router`: instance du routeur nécessaire que si vous utilisez une `loginRoute` avec `loginRouteIsInternal` à `true`.
- `serverCAS`: URL du serveur CAS (cf. le point `serverCAS` ci-dessus).

### login

Permet de lancer la méthode `login` du plugin.

La fonction prend comme paramètre un objet avec les attributs suivants :
- `jwtServerUrl` : URL du serveur qui gère le token (cf. le point `jwtServerUrl` ci-dessus).
- `options`: options du plugin (cf. le point `Options` ci-dessus).
- `router`: instance du routeur nécessaire que si vous utilisez une `loginRoute` avec `loginRouteIsInternal` à `true`.
- `serverCAS`: URL du serveur CAS (cf. le point `serverCAS` ci-dessus).

