import type { Mock } from 'vitest'
import { describe, expect, test, vi } from 'vitest'
import { refresh } from '@/security'
import type { AxiosInstance } from 'axios'
import axios, { AxiosError, AxiosHeaders } from 'axios'
import { jwtServerUrl } from '~/general.fixtures'

describe('refresh', () => {
  test('if the axios instance is not defined, should throw an error', async () => {
    // @ts-ignore
    await expect(async () => await refresh(undefined, ''))
      .rejects
      .toThrow('You must provide the axios instance.')
  })

  test('if the jwtServerUrl is not defined, should throw an error', async () => {
    const axios = {} as AxiosInstance
    await expect(async () => await refresh(axios))
      .rejects
      .toThrow('You must provide the JWT Server URL.')
  })

  test('if axios sent an error, should throw a custom error', async () => {
    const errorMessage = 'Erreur 401'
    const _axios = axios.create({})
    _axios.post = vi.fn();
    (_axios.post as Mock).mockRejectedValueOnce(
      new AxiosError(
        'Unauthorized',
        '401',
        {
          headers: new AxiosHeaders(),
        },
        {},
        {
          data: errorMessage,
          status: 401,
          statusText: 'Unauthorized',
          headers: {},
          config: {
            headers: new AxiosHeaders(),
          },
        },
      ),
    )

    await expect(async () => await refresh(_axios, jwtServerUrl))
      .rejects
      .toThrow('Vue cas authentication plugin error : ' + errorMessage)
  })

  test('if axios is ok, should update the localStorage', async () => {
    const _axios = axios.create({})
    _axios.post = vi.fn();
    (_axios.post as Mock).mockImplementation(() => ({
      data: {
        access: 'jwt',
      },
    }))

    expect(localStorage.getItem('JWT__access__token')).toBeNull()

    await refresh(_axios, jwtServerUrl)

    expect(localStorage.getItem('JWT__access__token')).toBe('jwt')
  })
})
