import { AxiosInterceptorConfig } from '#/index'
import { expired, login, refresh } from '@/security'
import type { InternalAxiosRequestConfig } from 'axios'

export const axiosRequestInterceptor = (
  { jwtServerUrl, serverCAS, router, axios, options }: AxiosInterceptorConfig,
  inComposable = true
): (config: InternalAxiosRequestConfig) => Promise<InternalAxiosRequestConfig> => {
  return async (config: InternalAxiosRequestConfig): Promise<InternalAxiosRequestConfig> => {
    /**
     * If the URL contains "/token/" the plugin lets the request pass.
     * It's a security for axios call for login or refresh call.
     */
    if (config.url?.match(/\/token\//)) {
      return config
    }

    /**
     * The case where the access token is not in storage not supported because, in the interceptor,
     * we don't know if the request secure or not.
     *
     * The conditions chain in the request interceptor is required to temporise the verification.
     */
    if (localStorage.getItem('JWT__access__token') !== null) {
      if (expired(localStorage.getItem('JWT__access__token') as string)) {
        if (localStorage.getItem('JWT__refresh__token') !== null) {
          if (expired(localStorage.getItem('JWT__refresh__token') as string)) {
            await login({ router, options, jwtServerUrl, serverCAS })
          } else {
            await refresh(axios, jwtServerUrl)
          }
        } else {
          await login({ router, options, jwtServerUrl, serverCAS })
        }
      }

      config.headers.Authorization = `Bearer ${localStorage.getItem('JWT__access__token')}`
    } else {
      if (inComposable) {
        await login({ router, options, jwtServerUrl, serverCAS })
      }
    }

    return config
  }
}
