import type { App } from 'vue'
import type { RouteLocationRaw, Router } from 'vue-router'
import type { AxiosInstance, InternalAxiosRequestConfig } from 'axios'

export interface InstallOptions {
  appIsAllAuth?: boolean;
  authCasLogoutUrl?: string;
  hasFreeAPI?: boolean;
  jwtServerUrl?: string;
  loginRoute?: RouteLocationRaw;
  loginRouteIsInternal?: boolean;
  serverCAS?: string;
  tokenErrorRoutes?: {
    default: string;
    [status: string]: string;
  }
}

export interface InstallConfig {
  axios: AxiosInstance;
  options?: InstallOptions;
  router: Router
}

export const CasAuthentication: {
  install(app: App, options: InstallConfig): void;
}

export interface LoginConfig {
  jwtServerUrl?: string,
  options?: InstallOptions,
  router?: Router,
  serverCAS?: string,
}

export interface AxiosInterceptorConfig {
  axios: AxiosInstance,
  jwtServerUrl: string,
  options?: InstallOptions,
  router: Router,
  serverCAS?: string,
}

export const useCasAuthentication: () => {
  axiosRequestInterceptor: (config: AxiosInterceptorConfig, inComposable = true) => (config: InternalAxiosRequestConfig) => Promise<InternalAxiosRequestConfig>;
  login: (config: LoginConfig) => Promise<void>;
}

declare module '@vue-unistra/formbuilder' {
  export const CasAuthentication: {
    install(app: App, options: InstallConfig): void;
  }
}
